<?php

namespace Drupal\media_fotoweb\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\media_fotoweb\FotowebLoginManagerInterface;
use Drupal\media_fotoweb\FotowebWidgetTrait;
use Drupal\media_library\Form\AddFormBase;
use Drupal\media_library\MediaLibraryUiBuilder;
use Drupal\media_library\OpenerResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Creates a form to create media entities from Fotoweb.
 */
class FotowebBrowserForm extends AddFormBase {

  use FotowebWidgetTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The Fotoweb login manager.
   *
   * @var \Drupal\media_fotoweb\FotowebLoginManagerInterface
   */
  protected $loginManager;

  /**
   * Constructs a AddFormBase object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\media_library\MediaLibraryUiBuilder $library_ui_builder
   *   The media library UI builder.
   * @param \Drupal\media_library\OpenerResolverInterface $opener_resolver
   *   The opener resolver.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory interface.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\media_fotoweb\FotowebLoginManagerInterface $login_manager
   *   The Fotoweb login manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, MediaLibraryUiBuilder $library_ui_builder, OpenerResolverInterface $opener_resolver = NULL, ConfigFactoryInterface $config_factory, AccountInterface $current_user, FotowebLoginManagerInterface $login_manager) {
    parent::__construct($entity_type_manager, $library_ui_builder, $opener_resolver);
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->loginManager = $login_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('media_library.ui_builder'),
      $container->get('media_library.opener_resolver'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('media_fotoweb.login_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'media_fotoweb_add_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function buildInputElement(array $form, FormStateInterface $form_state) {
    $config = $this->configFactory->get('media_fotoweb.settings');

    $form['selection_widget'] = [
      '#type' => 'markup',
      '#markup' => $this->buildEmbeddedSelectionWidget(),
    ];
    $form['fotoweb_selected'] = [
      '#type' => 'hidden',
    ];

    $form['selection_widget']['#attached']['library'][] = 'media_fotoweb/selection_widget';
    $form['selection_widget']['#attached']['drupalSettings']['media_fotoweb']['host'] = $config->get('server');

    // Create a visual hidden submit button. The asset selection will happen on
    // click.
    $form['actions']['submit'] = [
      '#value' => $this->t('Submit'),
      '#type' => 'submit',
      '#submit' => ['::addButtonSubmit'],
      '#ajax' => [
        'callback' => '::updateWidget',
        'wrapper' => 'media-library-wrapper',
      ],
      '#attributes' => [
        'class' => ['visually-hidden']
      ],
    ];
    return $form;
  }

  /**
   * Submit handler for the add button.
   *
   * @param array $form
   *   The form render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function addButtonSubmit(array $form, FormStateInterface $form_state) {
    $media = $this->prepareEntities($form, $form_state);
    // Any ID can be passed to the form, so we have to check access.
    $form_state->set('current_selection', array_filter($media, function ($media_item) {
      return $media_item->access('view');
    }));
    $form_state->setRebuild();
  }

}
